@@ -23,31 +23,33 @@
 #define SSH_NOIDENTIFSTR 279
 #define SSH_BADPROTOCOLIDENTIF 280
 #define SSH_BADPROTOCOLIDENTIF_SUFF 281
-#define SSH_DISCONNECT_PREF 282
-#define SSH_PREAUTH_SUFF 283
-#define DOVECOT_IMAP_LOGINERR_PREF 284
-#define DOVECOT_IMAP_LOGINERR_SUFF 285
-#define UWIMAP_LOGINERR 286
-#define CYRUSIMAP_SASL_LOGINERR_PREF 287
-#define CYRUSIMAP_SASL_LOGINERR_SUFF 288
-#define CUCIPOP_AUTHFAIL 289
-#define EXIM_ESMTP_AUTHFAIL_PREF 290
-#define EXIM_ESMTP_AUTHFAIL_SUFF 291
-#define SENDMAIL_RELAYDENIED_PREF 292
-#define SENDMAIL_RELAYDENIED_SUFF 293
-#define SENDMAIL_AUTHFAILURE_PREF 294
-#define SENDMAIL_AUTHFAILURE_SUFF 295
-#define POSTFIX_NO_AUTH_PREF 296
-#define POSTFIX_SASL_LOGINERR_PREF 297
-#define POSTFIX_SASL_LOGINERR_SUFF 298
-#define FREEBSDFTPD_LOGINERR_PREF 299
-#define FREEBSDFTPD_LOGINERR_SUFF 300
-#define PROFTPD_LOGINERR_PREF 301
-#define PROFTPD_LOGINERR_SUFF 302
-#define PUREFTPD_LOGINERR_PREF 303
-#define PUREFTPD_LOGINERR_SUFF 304
-#define VSFTPD_LOGINERR_PREF 305
-#define VSFTPD_LOGINERR_SUFF 306
+#define SSH_BADKEX_PREF 282
+#define SSH_BADKEX_SUFF 283
+#define SSH_DISCONNECT_PREF 284
+#define SSH_PREAUTH_SUFF 285
+#define DOVECOT_IMAP_LOGINERR_PREF 286
+#define DOVECOT_IMAP_LOGINERR_SUFF 287
+#define UWIMAP_LOGINERR 288
+#define CYRUSIMAP_SASL_LOGINERR_PREF 289
+#define CYRUSIMAP_SASL_LOGINERR_SUFF 290
+#define CUCIPOP_AUTHFAIL 291
+#define EXIM_ESMTP_AUTHFAIL_PREF 292
+#define EXIM_ESMTP_AUTHFAIL_SUFF 293
+#define SENDMAIL_RELAYDENIED_PREF 294
+#define SENDMAIL_RELAYDENIED_SUFF 295
+#define SENDMAIL_AUTHFAILURE_PREF 296
+#define SENDMAIL_AUTHFAILURE_SUFF 297
+#define POSTFIX_NO_AUTH_PREF 298
+#define POSTFIX_SASL_LOGINERR_PREF 299
+#define POSTFIX_SASL_LOGINERR_SUFF 300
+#define FREEBSDFTPD_LOGINERR_PREF 301
+#define FREEBSDFTPD_LOGINERR_SUFF 302
+#define PROFTPD_LOGINERR_PREF 303
+#define PROFTPD_LOGINERR_SUFF 304
+#define PUREFTPD_LOGINERR_PREF 305
+#define PUREFTPD_LOGINERR_SUFF 306
+#define VSFTPD_LOGINERR_PREF 307
+#define VSFTPD_LOGINERR_SUFF 308
 #ifdef YYSTYPE
 #undef  YYSTYPE_IS_DECLARED
 #define YYSTYPE_IS_DECLARED 1
