@@ -28,11 +28,11 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "fwalls/fw.h"
 #include "parser/parser.h"
 #include "simclist.h"
 #include "sshguard.h"
 #include "sshguard_blacklist.h"
-#include "sshguard_fw.h"
 #include "sshguard_log.h"
 #include "sshguard_logsuck.h"
 #include "sshguard_options.h"
@@ -76,7 +76,7 @@ static void sigfin_handler();
 static void finishup(void);
 
 /* load blacklisted addresses and block them (if blacklist enabled) */
-static void process_blacklisted_addresses();
+static void blacklist_load_and_block();
 /* handle an attack: addr is the author, addrkind its address kind, service the attacked service code */
 static void report_address(attack_t attack);
 /* cleanup false-alarm attackers from limbo list (ones with too few attacks in too much time) */
@@ -84,6 +84,23 @@ static void purge_limbo_stale(void);
 /* release blocked attackers after their penalty expired */
 static void *pardonBlocked();
 
+static void my_pidfile_create() {
+    FILE *p = fopen(opts.my_pidfile, "w");
+    if (p == NULL) {
+        sshguard_log(LOG_ERR, "Failed to create pid file: %m");
+        exit(73);
+    }
+
+    fprintf(p, "%d\n", (int)getpid());
+    fclose(p);
+}
+
+static void my_pidfile_destroy() {
+    if (unlink(opts.my_pidfile) != 0) {
+        sshguard_log(LOG_ERR, "Failed to remove pid file: %m");
+    }
+}
+
 /**
  * Fill 'buf' with a line from a log source and set the 'source_id' pointer.
  * Return 0 on success and -1 on failure.
@@ -126,13 +143,19 @@ int main(int argc, char *argv[]) {
         exit(64);
     }
 
+    if (opts.my_pidfile != NULL) {
+        my_pidfile_create();
+        atexit(my_pidfile_destroy);
+    }
+
     if (fw_init() != FWALL_OK) {
         sshguard_log(LOG_ERR, "Failed to initialize firewall");
         exit(69);
     }
 
-    // Load blacklist and block listed addresses.
-    process_blacklisted_addresses();
+    if (opts.blacklist_filename != NULL) {
+        blacklist_load_and_block();
+    }
 
     /* termination signals */
     signal(SIGTERM, sigfin_handler);
@@ -191,21 +214,21 @@ int main(int argc, char *argv[]) {
 
 void log_block(attacker_t *tmpent, attacker_t *offenderent) {
     char *time_msg;
-    if (tmpent->pardontime > 0) {
-        if (asprintf(&time_msg, "for %lld secs",
-                     (long long)tmpent->pardontime) < 0) {
+    const time_t time = tmpent->pardontime;
+    if (time > 0) {
+        if (asprintf(&time_msg, "for %lld secs", (long long)time) < 0) {
             abort();
         }
     } else {
         time_msg = "forever";
     }
-    sshguard_log(LOG_NOTICE, "%s: blocking %s (%u attacks in %lld "
+    sshguard_log(LOG_NOTICE, "Blocking %s %s (%u attacks in %lld "
                              "secs, after %d abuses over %lld secs)",
                  tmpent->attack.address.value, time_msg, tmpent->numhits,
                  (long long)(tmpent->whenlast - tmpent->whenfirst),
                  offenderent->numhits,
                  (long long)(offenderent->whenlast - offenderent->whenfirst));
-    if (tmpent->pardontime > 0) {
+    if (time > 0) {
         // Free time message only if previously allocated.
         free(time_msg);
     }
@@ -234,7 +257,7 @@ static void report_address(attack_t attack) {
     tmpent = list_seek(& hell, & attack.address);
     pthread_mutex_unlock(& list_mutex);
     if (tmpent != NULL) {
-        sshguard_log(LOG_WARNING, "%s: should already have been blocked",
+        sshguard_log(LOG_WARNING, "%s has already been blocked",
                 attack.address.value);
         return;
     }
@@ -304,14 +327,13 @@ static void report_address(attack_t attack) {
     } else {
         /* compute blocking time wrt the "offensiveness" */
         for (unsigned int i = 0; i < offenderent->numhits; i++) {
-            tmpent->pardontime *= 1.5;
+            tmpent->pardontime *= 2;
         }
     }
     list_sort(& offenders, -1);
     log_block(tmpent, offenderent);
 
-    int ret = fw_block(attack.address.value,
-            attack.address.kind, attack.service);
+    int ret = fw_block(&attack);
     if (ret != FWALL_OK) {
         sshguard_log(LOG_ERR, "fw: failed to block (%d)", ret);
     }
@@ -337,43 +359,48 @@ static void purge_limbo_stale(void) {
     }
 }
 
-static void *pardonBlocked() {
-    time_t now;
+static void unblock_expired() {
     attacker_t *tmpel;
     int ret;
+    time_t now = time(NULL);
 
+    pthread_testcancel();
+    pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &ret);
+    pthread_mutex_lock(&list_mutex);
+
+    for (unsigned int pos = 0; pos < list_size(&hell); pos++) {
+        tmpel = list_get_at(&hell, pos);
+        /* skip blacklisted hosts (pardontime = infinite/0) */
+        if (tmpel->pardontime == 0)
+            continue;
+        /* process hosts with finite pardon time */
+        if (now - tmpel->whenlast > tmpel->pardontime) {
+            /* pardon time passed, release block */
+            sshguard_log(LOG_DEBUG, "Unblocking %s after %lld secs",
+                         tmpel->attack.address.value,
+                         (long long)(now - tmpel->whenlast));
+            ret = fw_release(&tmpel->attack);
+            if (ret != FWALL_OK) {
+                sshguard_log(LOG_ERR, "fw: failed to unblock (%d)", ret);
+            }
+            list_delete_at(&hell, pos);
+            free(tmpel);
+            /* element removed, next element is at current index (don't step
+             * pos) */
+            pos--;
+        }
+    }
+
+    pthread_mutex_unlock(&list_mutex);
+    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &ret);
+    pthread_testcancel();
+}
+
+static void *pardonBlocked() {
     while (1) {
         /* wait some time, at most opts.pardon_threshold/3 + 1 sec */
         sleep(1 + ((unsigned int)rand() % (1+opts.pardon_threshold/2)));
-        now = time(NULL);
-        pthread_testcancel();
-        pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &ret);
-        pthread_mutex_lock(& list_mutex);
-
-        for (unsigned int pos = 0; pos < list_size(&hell); pos++) {
-            tmpel = list_get_at(&hell, pos);
-            /* skip blacklisted hosts (pardontime = infinite/0) */
-            if (tmpel->pardontime == 0) continue;
-            /* process hosts with finite pardon time */
-            if (now - tmpel->whenlast > tmpel->pardontime) {
-                /* pardon time passed, release block */
-                sshguard_log(LOG_INFO, "%s: unblocking after %lld secs",
-                        tmpel->attack.address.value,
-                        (long long)(now - tmpel->whenlast));
-                ret = fw_release(tmpel->attack.address.value, tmpel->attack.address.kind, tmpel->attack.service);
-                if (ret != FWALL_OK) {
-                    sshguard_log(LOG_ERR, "fw: failed to unblock (%d)", ret);
-                }
-                list_delete_at(&hell, pos);
-                free(tmpel);
-                /* element removed, next element is at current index (don't step pos) */
-                pos--;
-            }
-        }
-        
-        pthread_mutex_unlock(& list_mutex);
-        pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &ret);
-        pthread_testcancel();
+        unblock_expired();
     }
 
     pthread_exit(NULL);
@@ -403,66 +430,24 @@ static void sigfin_handler(int sig) {
     exit(0);
 }
 
-static void process_blacklisted_addresses() {
-    list_t *blacklist;
-    const char **addresses;         /* NULL-terminated array of (string) addresses to block:  char *addresses[]  */
-    int *restrict service_codes;    /* array of service codes resp to the given addresses */
-
-    if (opts.blacklist_filename == NULL) {
-        // Return if blacklisting is not enabled.
-        return;
+static void block_list(list_t *list) {
+    list_iterator_start(list);
+    while (list_iterator_hasnext(list)) {
+        attacker_t *next = list_iterator_next(list);
+        fw_block(&next->attack);
     }
+    list_iterator_stop(list);
+}
 
-    blacklist = blacklist_load(opts.blacklist_filename);
+static void blacklist_load_and_block() {
+    list_t *blacklist = blacklist_load(opts.blacklist_filename);
     if (blacklist == NULL) {
-        perror("Could not open blacklist");
-        sshguard_log(LOG_ERR, "blacklist: could not open %s",
+        sshguard_log(LOG_ERR, "blacklist: could not open %s: %m",
                 opts.blacklist_filename);
         exit(66);
     }
 
-    /* blacklist enabled */
-    size_t num_blacklisted = list_size(blacklist);
-    sshguard_log(LOG_INFO, "blacklist: blocking %lu addresses",
-            (long unsigned int)num_blacklisted);
-    /* prepare to call fw_block_list() to block in bulk */
-    /* two runs, one for each address kind (but allocate arrays once) */
-    addresses = (const char **)malloc(sizeof(const char *) * (num_blacklisted+1));
-    service_codes = (int *restrict)malloc(sizeof(int) * num_blacklisted);
-    int addrkind;
-    for (addrkind = ADDRKIND_IPv4; addrkind != -1; addrkind = (addrkind == ADDRKIND_IPv4 ? ADDRKIND_IPv6 : -1)) {
-        /* extract from blacklist only addresses (and resp. codes) of type addrkind */
-        int i = 0;
-        list_iterator_start(blacklist);
-        while (list_iterator_hasnext(blacklist)) {
-            const attacker_t *bl_attacker = list_iterator_next(blacklist);
-            if (bl_attacker->attack.address.kind != addrkind)
-                continue;
-
-            // Trim trailing newline from strchr().
-            char *time_str = ctime(&bl_attacker->whenlast);
-            char *newline = strchr(time_str, '\n');
-            assert(newline != NULL);
-            *newline = '\0';
-            sshguard_log(LOG_DEBUG,
-                    "blacklist: loaded %s (ip%d) on service %d: %s",
-                    bl_attacker->attack.address.value,
-                    bl_attacker->attack.address.kind,
-                    bl_attacker->attack.service, time_str);
-            addresses[i] = bl_attacker->attack.address.value;
-            service_codes[i] = bl_attacker->attack.service;
-            ++i;
-        }
-        list_iterator_stop(blacklist);
-        if (i == 0) continue;
-        /* terminate array list */
-        addresses[i] = NULL;
-        /* do block addresses of this kind */
-        if (fw_block_list(addresses, addrkind, service_codes) != FWALL_OK) {
-            sshguard_log(LOG_ERR, "blacklist: failed to block addresses");
-        }
-    }
-    /* free temporary arrays */
-    free(addresses);
-    free(service_codes);
+    sshguard_log(LOG_INFO, "blacklist: blocking %u addresses",
+            (unsigned int)list_size(blacklist));
+    block_list(blacklist);
 }
