@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007,2008 Mij <mij@bitchx.it>
+ * Copyright (c) 2007,2008,2010 Mij <mij@sshguard.net>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -21,20 +21,12 @@
 #ifndef SSHGUARD_H
 #define SSHGUARD_H
 
-#include "sshguard_addresskind.h"
+/* default "weight" of an attack */
+#define DEFAULT_ATTACKS_DANGEROUSNESS           10
 
-/* these may be changed with runtime options! */
-
-/* default: minimum seconds after which unblocking a blocked IP. Max is (min*3/2) */
-#define DEFAULT_PARDON_THRESHOLD    (7 * 60)
-/* default number of hits after which blocking an IP (inclusive) */
-#define DEFAULT_ABUSE_THRESHOLD     4
-/* default seconds after which forgiving a cracker candidate */
-#define DEFAULT_STALE_THRESHOLD     (20 * 60)
-/* default number of abuses (blocked) from which the attacker gets blacklisted and blocked permanently */
-#define DEFAULT_BLACKLIST_THRESHOLD 3
-
-/* maximum number of recent offenders to retain in memory at once */
-#define MAX_OFFENDER_ITEMS      15
+/* maximum number of files polled */
+#define MAX_FILES_POLLED        35
+/* maximum file polling interval when logs are idle (millisecs) */
+#define MAX_LOGPOLL_INTERVAL    1000
 
 #endif
