@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007,2008 Mij <mij@bitchx.it>
+ * Copyright (c) 2007,2008 Mij <mij@sshguard.net>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -20,13 +20,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <time.h>
-#include <sys/wait.h>
 #include <string.h>
-#include <errno.h>
+#include <sys/wait.h>
 #include <unistd.h>
-#include <simclist.h>
 
+#include "simclist.h"
 #include "sshguard_log.h"
 #include "sshguard_procauth.h"
 
@@ -36,7 +34,7 @@ typedef struct {
     pid_t current_pid;
 } procpid;
 
-size_t procpid_meter(const void *el) {
+size_t procpid_meter() {
     return sizeof(procpid);
 }
 
@@ -46,17 +44,12 @@ list_t proclist;
 static pid_t procauth_getprocpid(char *filename);
 static int procauth_ischildof(pid_t child, pid_t parent);
 
-
-
-int procauth_init() {
-    /* assume random number generator already seeded */
+void procauth_init() {
     list_init(&proclist);
     list_attributes_copy(&proclist, procpid_meter, 1);
-
-    return 0;
 }
 
-int procauth_fin() {
+void procauth_fin() {
     procpid *pp;
 
     /* free filenames */
@@ -69,7 +62,6 @@ int procauth_fin() {
 
     /* destroy the list itself */
     list_destroy(&proclist);
-    return 0;
 }
 
 int procauth_addprocess(char *conf) {
@@ -79,7 +71,7 @@ int procauth_addprocess(char *conf) {
 
     /* ensure the filename length is within system limit */ 
     if (memchr(conf, '\0', FILENAME_MAX) == NULL)
-	return -1;
+    return -1;
 
     /* conf format:     service_code:pid_filename   */
     if (sscanf(conf, "%d:%s", &srvcode, pidfilename) != 2)
@@ -160,7 +152,8 @@ static pid_t procauth_getprocpid(char *filename) {
     }
 
     if (fscanf(pf, "%d", &pid) != 1) {
-        sshguard_log(LOG_INFO, "pid file '%s' malformed. Expecting one pid.");
+        sshguard_log(LOG_INFO, "pid file '%s' malformed. Expecting one pid.", filename);
+        fclose(pf);
         return -1;
     }
     fclose(pf);
@@ -169,62 +162,64 @@ static pid_t procauth_getprocpid(char *filename) {
 }
 
 static int procauth_ischildof(pid_t child, pid_t parent) {
-    char pattern[30];
-    int retA, retB;
-    pid_t pidA, pidB;
-    int ps2grep[2];
+    int ischild;
+    int ret;
+    int curchild, curparent;
+    char mystring[20];
+    int ps2me[2];
+    pid_t pid;
+    FILE *psout;
 
-    sprintf(pattern, "%d[[:space:]]+%d", child, parent);
 
-    /* pipe from ps out to grep */
-    if (pipe(ps2grep) != 0) {
-        sshguard_log(LOG_ERR, "In pipe(): %s.", strerror(errno));
+    sshguard_log(LOG_DEBUG, "Testing if %d is child of %d.", (int)child, (int)parent);
+
+    if (pipe(ps2me) == -1) {
+        sshguard_log(LOG_ERR, "Can't create pipe! %s.", strerror(errno));
         return 0;
     }
 
-    if ((pidA = fork()) == 0) {
-        /* in child A (for grep) */
-        close(0); close(1); close(2);
-            
-        dup2(ps2grep[0], 0);
+    /* execute ps command, pipe result to us */
+    if ((pid = fork()) == 0) {
+        /* child */
+        close(0);
+        dup2(ps2me[1], 1);
 
-        /* "grep -qE '123[[:space:]]+456'" */
-        execlp("grep", "grep", "-qE", pattern, NULL);
-        sshguard_log(LOG_ERR, "Could not run grep: %s.", strerror(errno));
-        _Exit(250);
-    }
-
-    if ((pidB = fork()) == 0) {
-        /* in child B (for ps) */
-        close(0); close(1); close(2);
-
-        dup2(ps2grep[1], 1);
-
-        /* "ps axo pid,ppid" */
+        sshguard_log(LOG_DEBUG, "Running 'ps axo pid,ppid'.");
         execlp("ps", "ps", "axo", "pid,ppid", NULL);
-        sshguard_log(LOG_ERR, "Could not run ps: %s.", strerror(errno));
-        _Exit(250);
+
+        sshguard_log(LOG_ERR, "Unable to run 'ps axo pid,ppid': %s.", strerror(errno));
+        exit(-1);
     }
 
-    /* parent: wait A and B */
-    waitpid(pidA, &retA, 0);
-    retA = WEXITSTATUS(retA);
-    waitpid(pidB, &retB, 0);
-    retB = WEXITSTATUS(retB);
+    /* father */
+    close(ps2me[1]);
 
-    sshguard_log(LOG_DEBUG, "Run \"ps axo pid,ppid\" -> \"grep -qE '%s'\", returned %d and %d.", pattern, retB, retA);
+    /* read lines returned the ps' output, and match pschild_re */
+    psout = fdopen(ps2me[0], "r");
+    if (psout == NULL) return 0;
 
-    if (retA != 0 || retB > 2) return 0;    /* error executing a tool */
+    ischild = 0;
+    while (fgets(mystring, sizeof(mystring), psout) != NULL) {
+        /* sshguard_log(LOG_DEBUG, "Testing child/parent line: '%s' --> %d@%d", mystring, curchild, curparent); */
+        sscanf(mystring, " %d %d ", & curchild, & curparent);
+        if (curchild == child && curparent == parent) {
+            ischild = 1;
+            break;
+        }
+    }
+
+    waitpid(pid, & ret, 0);
+    fclose(psout);
+    if (! WIFEXITED(ret) || WEXITSTATUS(ret) != 0) {
+        sshguard_log(LOG_ERR, "ps command failed to run.");
+        return 0;
+    }
     
-    /* derive final value to return */
-    switch (retB) {
-        case 1: /* child-parent association NOT FOUND */
-            return -1;
-        case 0: /* child-parent assured FOUND */
-            return 1;
-    }
+    sshguard_log(LOG_INFO, "Process %d %s child of %d.", (int)child, (ischild ? "is" : "is not"), (int)parent);
+    /* return YES or NO answer */
+    if (ischild)
+        return 1;
 
-    /* an error occurred during verification */
-    return 0;
+    return -1;
 }
 
